/** *****************************************************
 * Copyright (C) HRider in motion S.L.
 *
 * This file is part of HRider.
 *
 * HRider can not be copied and/or distributed without the express
 * permission of HRider in motion S.L.
 ****************************************************** */
package net.hrider.api.usecases;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;
import net.hrider.api.client.ApiException;
import net.hrider.api.model.Appraisal;
import net.hrider.api.model.Company;
import net.hrider.api.model.Link;
import net.hrider.api.model.Report;
import net.hrider.api.model.ReportsCollection;

/**
 * Example to get reports for an appraisal.
 *
 * @author ahc (aina@hrider.net)
 */
public class GetReports extends AbstractUseCase {

    private final long companyId;
    private final long appraisalId;
    private final String outputDir;

    private static final Logger LOG = Logger.getLogger(GetReports.class.getName());

    public GetReports(long companyId, long appraisalId, String outputDir) {
        this.companyId = companyId;
        this.appraisalId = appraisalId;
        this.outputDir = outputDir;
    }
    
    @Override
    public void doRun() throws ApiException {
        // Find company
        Link companyLink = root.getLinks().getCompanies().build("" + companyId);
        Company company = service.get(Company.class, companyLink);

        // Get appraisal
        Link appraisalLink = company.getLinks().getAppraisals().build("" + appraisalId);
        Appraisal appraisal = service.get(Appraisal.class, appraisalLink);

        // Get reports
        ReportsCollection collection = service.get(ReportsCollection.class, appraisal.getLinks().getReports());        
        List<Report> reports = collection.getEmbedded().getReports();
        printReports(reports);
        
        // Get Excel file and save it
        Report report = collection.getEmbedded().findReport("feedback-results");
        InputStream in = service.get(InputStream.class, report.getLinks().getSelf(), 
                net.hrider.api.Constants.MIMETYPE_EXCEL);
        
        try (FileOutputStream fout = new FileOutputStream(new File(outputDir, "feedback-results.xls"))) {
            saveToFile(in, fout);
        } catch (IOException ex) {
            LOG.log(Level.SEVERE, "error saving file", ex);
        }
    }

    /**
     * Main method. Accepts 3 arguments, the company ID, the appraisal ID and
     * the output dir to save the excel file result.
     * 
     * @param args company ID (long), appraisal ID (long) and outputDir (String).
     */
    public static void main(String[] args) {
        if (args.length == 3) {
            try {
                GetReports uc = new GetReports(Long.parseLong(args[0]), Long.parseLong(args[1]), args[2]);
                uc.run();
            } catch (NumberFormatException ex) {
                printUsage();
            }
        } else {
            printUsage();
        }
    }
        
    private static void printUsage() {
        System.err.println("Usage: companyId appraisalId outputDir");
        System.err.println("companyId: the company id (long). \n"
                + "appraisalId: the appraisal id (long). \n"
                + "outputDir: path to the output directory to save the file.");
    }

}
